/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** grids.h code for the boundaries of the system
****************************************************************************/

#ifndef GRIDS_H_
#define GRIDS_H_

#include <vector>
#include <memory>
#include <algorithm>
#include "isac.h"

namespace isac {

/* Creates element set forming a uniform 3-D grid.
 * Accepts divisions along each axis, dimension along each axis,
 * and default Element initialization value. */
std::vector<Element_type> create_grid(int divx, int divy, int divz,
	double size_x, double size_y, double size_z, double default_value = 0.0);

/* Creates element set forming a uniform 2-D grid */
std::vector<Element_type> create_grid(int divx, int divy,
	double size_x, double size_y, double default_value = 0.0);

}

#endif
