/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** init.h
****************************************************************************/

#ifndef TMODEL_INIT_H_
#define TMODEL_INIT_H_

#include <vector>
#include <cmath>
#include <ctime>
#include <cassert>
#include <fstream>
#include "hoct_model.h"
#include "isac.h"

/* Initialize the "isot"(isothermal grid) and "cont"(constant temperature 
 * grid) grids */
void isotcont_init(par_grid & isot, par_grid & cont);

/* Initiate the hoct_contour data structure based on the "root" grid */
/* If the hoct_contour only has 4 levels, then hoct_cont_init() function
 * call will generate the correct hoct_contour data structure. Otherwise,
 * hoct_contour_update() function is called to generate the levels other
 * than the first 4 levels */
/* The information of the first 4 levels: which are preserved the same after
 * initiation, even though further refinement is conducted for some elements.
 * Level 0: One grid only, which reparesents the whole chip package;
 * Level 1: Grid number equal to the number of components. Each grid represents
 * one component; 
 * Level 3: Grid number equal to the number of material and interface layers.
 * Each grid represents one material layer or one interface layer;
 * Level 4: Grid representing all the elements after the initial partition.
 * Grids in Level 1 are direct children of Level 0 grid;
 * Grids in Level 2 are direct children of grids in Level 1;
 * Grids in Level 3 are direct children of grids in Level 2. */
void hoct_contour_init(std::vector<std::vector<grid_contour> > & hoct_contour, 
    par_grid * root, par_grid * isot, par_grid * cont);

/* Generate the hoct_contour data structure (levels greater than 3) based on 
 * the partition of the "root" */ 
void hoct_contour_update(std::vector<std::vector<grid_contour> > & hoct_contour, 
    par_grid * root/*, par_grid * isot, par_grid * cont*/);

/* Generate the all the neighbor information for the grid_contours in level 
 * 3 (This level of grid_contours never change) hoct_contour data structure
 * (based on the neighbor information from the "root") */
void contour_nei_init(par_grid * root, std::vector<std::vector<grid_contour> >
	& hoct_contour, int init_level);

#endif
