/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** nhar.h: The NHAR class
****************************************************************************/

#ifndef NHAR_H_
#define NHAR_H_

#include <iostream>
#include <sstream>
#include <vector>
#include <ctime>
#include "hoct_model.h"
#include "init.h"
#include "isac.h"

namespace isac {
/*===========================================================================*/
/* The class for 3D frequency domain dynamic thermal modeling using nhar
 * technique */
class nhar{
		
public:

    //* Constructor of nhar class. */
	nhar(RC_network_type & AGC);

    // Expand the input in an infinite Taylor series around zero frequency, and
    // truncate to n terms
    void taylor_expand(double duration, std::vector<double> & power);

    //Generate the projection matrix Q by employing algorithm 1
    void nhar_projection(RC_network_type& AGC);

    //Project the matrices using Q to get the reduced system
    void project_system(std::vector<double> & power,
        std::vector<double> & init_temp, RC_network_type & AGC);


    //Solve the system for x~(s)
    void solve(std::vector<std::vector<double> > & all_temp,
        std::vector<double> & power, double duration,
        double tempreport_interval);

    //Compute the temperatures by obtaining x(s) from x~(s)
    void temp_compute(std::vector<double> & temp);

	int & moments() {return moments_;}
	int & trunc_order() {return trunc_order_;}
	int & N() {return N_;}

private:

	int N_;	
	int moments_;
    int trunc_order_;

	//------inputs to NHAR Algorithm 1------//
	std::vector<double> UM_matrix_;
	
    //-------- projection matrix ---------//
    std::vector<double> Q_matrix_;

   //------outputs of NHAR Algorithm------//
   std::vector<double> reduced_G_;
   std::vector<double> reduced_C_;
   std::vector<double> reduced_B_;

   std::vector<double> init_reduced_temp_;
   std::vector<double> reduced_temp_;
   
};
} /* namespace isac */

#endif
