/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** output.cc: prints output data to file
****************************************************************************/

#include "header.h"

/*==========================================================================*/
//Output function for the entire system
void Device_structure::output(string argv) {

    ofstream fout, fout_stat, fout_plot;

    double peak_temp=0;
    int peak_temp_x,peak_temp_y,peak_temp_z;
    peak_temp_x=peak_temp_y=peak_temp_z=-1;
    cout<<" argv "<<argv<<endl;	
	//if output filename is specified, use it
    if(argv!="") {
        fout.open(argv.c_str());
        string meaty = argv;
		string plot = "_plot";
		string stat = "_stat";

        fout_plot.open((meaty+plot).c_str());
        fout_stat.open((meaty+stat).c_str());

    }
    else { //else use default name "output.txt"
        fout.open("output.txt");
        fout_plot.open("output_plot.txt");
	    fout_stat.open("stat_output.txt");
    }
	//update each Fourier cell temperature with the temperature from the BTE
	//blocks
	update_fourier_t_cells();

	//output the results (output Fourier cell temperatures since they contain 
	//all the Fourier and BTE results)
	int cells_x = substrate.f_cells_x;
	int cells_y = substrate.f_cells_y;
	int cells_z = substrate.f_cells_z;
	int flag = 0; //flag to let it know it is in the BTE region
	int flagger = 0; //another flag to say it was not in any bte region

    fout_stat<<substrate.del_x<<endl;
    fout_stat<<substrate.del_y<<endl;
    fout_stat<<substrate.del_z<<endl;
    fout_stat<<substrate.f_cells_x<<endl;
    fout_stat<<substrate.f_cells_y<<endl;
    fout_stat<<substrate.f_cells_z<<endl;
    // ---------- For plotting -------- //
    fout_plot<<substrate.f_cells_x-2+substrate.tbound_x_0+substrate.tbound_x_end;
    fout_plot<<" ";
    fout_plot<<substrate.f_cells_y-2+substrate.tbound_y_0+substrate.tbound_y_end;
    fout_plot<<" ";
    fout_plot<<substrate.f_cells_z-2+substrate.tbound_z_0+substrate.tbound_z_end;
    fout_plot<<endl;
    // -------- End for plotting ------ //

	for(int i=1-substrate.tbound_x_0;i<(cells_x-1+substrate.tbound_x_end);i++) {
        for(int j=1-substrate.tbound_y_0;
									j<(cells_y-1+substrate.tbound_y_end);j++) {
            for(int k=1-substrate.tbound_z_0;
									k<(cells_z-1+substrate.tbound_z_end);k++) {

				flagger = 0;
				int siize = bte_system.size();
				//for all the bte regions
				for(int v=0; v<siize; v++) {
					//if its inside the BTE region
					if((i>=(bte_coords[v].x-1) &&j>=(bte_coords[v].y-1) &&
								k>=(bte_coords[v].z-1)) &&
						(i<=(bte_coords[v].x+bte_coords[v].cells_x-2) &&
						j<=(bte_coords[v].y+bte_coords[v].cells_y-2)&&
						k<=(bte_coords[v].z+bte_coords[v].cells_z-2)) ){
						
						//if this is a BTE boundary and it is touching a
						//fourier region, let the fourier print the accurate
						//cell info

						int print = 1;

						//if it is a west fourier boundary
						if((i==(bte_coords[v].x-1))&&
										(bte_system[v].bte_boundaries[1]==1)) {
							print = 0;
						}
						//east
						if((i==(bte_coords[v].x+bte_coords[v].cells_x-2))&&
                                        (bte_system[v].bte_boundaries[0]==1)) {
                            print = 0;
                        }
						//if it is a south fourier boundary
                        if((j==(bte_coords[v].y-1))&&
                                        (bte_system[v].bte_boundaries[3]==1)) {
                            print = 0;
                        }
                        //north
                        if((j==(bte_coords[v].y+bte_coords[v].cells_y-2))&&
                                        (bte_system[v].bte_boundaries[2]==1)) {
                            print = 0;
                        }
						//if it is a bottom fourier boundary
                        if((k==(bte_coords[v].z-1))&&
                                        (bte_system[v].bte_boundaries[5]==1)) {
                            print = 0;
                        }
                        //top
                        if((k==(bte_coords[v].z+bte_coords[v].cells_z-2))&&
                                        (bte_system[v].bte_boundaries[4]==1)) {
                            print = 0;
                        }

						//if the bte region should print it, print it
						if(print == 1) {

							if(flag == 0) {
								fout<<"/* ";
							}				
							//print the temperature value of this cell

                            // --------- For plotting ------------- //
                            fout_plot<<substrate.cen_x[i]<<" "<<
                                       substrate.cen_y[j]<<" "<<
                                       substrate.cen_z[k]<<" ";
							fout_plot<<bte_system[v].cell[i-bte_coords[v].x+1]
						[j-bte_coords[v].y+1][k-bte_coords[v].z+1].T_<<" ";

                        //set the temperature of the bte cell to the fourier cell
                        substrate.f_cell[i][j][k].T_ = bte_system[v].cell[i-bte_coords[v].x+1]
                        [j-bte_coords[v].y+1][k-bte_coords[v].z+1].T_;

                            fout_plot<<endl;
                            // ------- End for plotting ----------- //

							fout<<bte_system[v].cell[i-bte_coords[v].x+1]
						[j-bte_coords[v].y+1][k-bte_coords[v].z+1].T_<<" ";
							if(bte_system[v].cell[i-bte_coords[v].x+1]
						[j-bte_coords[v].y+1][k-bte_coords[v].z+1].T_>peak_temp)
							{
							    peak_temp=bte_system[v].cell[i-bte_coords[v].x+1]
						[j-bte_coords[v].y+1][k-bte_coords[v].z+1].T_;
							    peak_temp_x=i;
							    peak_temp_y=j;
							    peak_temp_z=k;
								
							}

							flag = 1;
							flagger = 1;
						}
					}
				}
				if(flagger==0) {
					if(flag == 1) {
                       	fout<<"*/ ";
                    }
					flag = 0;

                    // --------- For plotting ------------- //
                    fout_plot<<substrate.cen_x[i]<<" "<<
                               substrate.cen_y[j]<<" "<<
                               substrate.cen_z[k]<<" ";
					fout_plot<<substrate.f_cell[i][j][k].T_<<" ";
                    fout_plot<<endl;
                    // -------- End for plotting --------- //
					fout<<substrate.f_cell[i][j][k].T_<<" ";
					if(substrate.f_cell[i][j][k].T_>peak_temp)
					{
					    peak_temp=substrate.f_cell[i][j][k].T_;
					    peak_temp_x=i;	
					    peak_temp_y=j;
					    peak_temp_z=k;
					}
                }
            }
            fout<<endl;
        }
        fout<<endl;
    }
	fout.close();
    fout_plot.close();

    //print some statistics in the _stat file
    fout_stat<<"Peak Temperature = "<<peak_temp<<endl;
    fout_stat<<"Location of peak temperature:"<<endl;
    fout_stat<<"x="<<substrate.cen_x[peak_temp_x]<<endl;
    fout_stat<<"y="<<substrate.cen_y[peak_temp_y]<<endl;
    fout_stat<<"z="<<substrate.cen_z[peak_temp_z]<<endl;

    //New statistics
    int bte_regions = bte_system.size();
    int total_bte_cells_x, total_bte_cells_y, total_bte_cells_z;
    total_bte_cells_x=total_bte_cells_y=total_bte_cells_z=0;

    for(int bte=0;bte<bte_regions;bte++)
    {
        total_bte_cells_x = total_bte_cells_x + bte_system[bte].cells_x-2;
        total_bte_cells_y = total_bte_cells_y + bte_system[bte].cells_y-2;
        total_bte_cells_z = total_bte_cells_z + bte_system[bte].cells_z-2;
    }
    fout_stat<<"Total number of BTE cells = "<<total_bte_cells_x*
                                               total_bte_cells_y*
                                               total_bte_cells_z<<endl;
        
    
    fout_stat<<"Total number of Fourier Cells = "<<(substrate.f_cells_x*
                                                    substrate.f_cells_y*  
                                                    substrate.f_cells_z)-
                                                    (total_bte_cells_x*
                                                    total_bte_cells_y*
                                                    total_bte_cells_z)<<endl;

    fout_stat.close();


	double num=0;
    for(int i=1; i<cells_x-1; i++) {
        for(int j=1; j<cells_y-1; j++) {
            for(int k=1; k<cells_z-1; k++) {

                num=num+substrate.f_cell[i][j][k].Qprime;
            }
        }
    }
}


