/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** solver.h: the solver class in ISAC
****************************************************************************/

#ifndef SOLVER_H_
#define SOLVER_H_

#include "hoct_model.h"
#include "util.h"

class solver_type {

public :

    solver_type();
	solver_type(RC_network_type& AGC);
	~solver_type();

    /* Iterative Gauss Seidel method to calculate T in AT=P */
	void ssmoother(std::vector<std::vector<int> > & A, 
		std::vector<double> & temp, std::vector<double> & power, 
		double err_thre, int index);

    /* Iterative Gauss Seidel method to calculate T in AT=P */
    /* Do the smoother inside the AGC accoring to the start and end id (only do
     * the smoother for those elements that reside inside {start_id, end_id} */
    void ssmoother( RC_network_type & AGC, std::vector<double>& temp,
        std::vector<double>& power, double err_thre, int start_id, int end_id);

    /* Iterative Gauss Seidel method to calculate T in AT=P */
    void ssmoother(RC_network_type & AGC,
		std::vector<double> & temp, std::vector<double> & power, 
		double err_thre);

    /* Iterative Gauss Seidel method to calculate T in AT=P */
    void ssmoother2(RC_network_type & AGC,
		std::vector<double> & temp, std::vector<double> & power, 
		double err_thre);

    /* Iterative Gauss Seidel method to calculate T in AT=P */
    void ssmoother(RC_network_type & AGC,
		std::vector<double> & temp, std::vector<double> & power, 
		double err_thre, int index);

private :
	
	std::vector<std::vector<double> > G_;
	std::vector<double> Gx_;
	std::vector<double> f_;
	std::vector<double> residue_;

};

#endif	
